//     !     
//  .         UniText, 
//    protocol.utx,          .

#pragma once
#include "kernel/protocolwrapper.h"

using namespace Uniteller::Framework::Kernel;
namespace Uniteller
{
  namespace CrossroadController
  {
    namespace Protocol
    {

      BEGIN_PROTOCOL(101, 1)
        
        #pragma region 
        enum class MyColorType
        {
          Green = 3,
          Red = 1,
          Yellow = 2
        };
        
        // 
        inline bool IsValidMyColorTypeValue(const int value)
        {
          return ((value >= 1)&&(value < 4));
        }
        
        inline MyColorType ConvertToMyColorType(const int value)
        {
          switch (value)
          {
            case 3:
              return MyColorType::Green;
            case 1:
              return MyColorType::Red;
            case 2:
              return MyColorType::Yellow;
            default: throw KernelException("    %d   MyColorType", value);
            }
        }
        
        #pragma endregion
        
        #pragma region   
        
        //
        BEGIN_MESSAGE(GetState,2,1)
          /// <summary>
          ///  
          /// </summary>
          void InitializeMessage()
          {
          }
          ARGUMENT(0, string, Device,  );
        END_MESSAGE(); // GetState

        BEGIN_MESSAGE(SetState,1,2)
          /// <summary>
          ///  
          /// </summary>
          void InitializeMessage()
          {
          }
          ARGUMENT(0, string, Device,  );
          ARGUMENT(1, string, DeviceState,  );
        END_MESSAGE(); // SetState

        
        BEGIN_WRAPPER()
          /// <param name="device"></param>
          void GetState(const string & device)
          {
            G1::GetState cmd;
            cmd.Device = device;
            m_RequestDispatcher->SendMessage(m_sModuleId.c_str(), cmd);
          }
          
          /// <param name="device"></param>
          /// <param name="deviceState"></param>
          void SetState(const string & device, const string & deviceState)
          {
            G1::SetState cmd;
            cmd.Device = device;
            cmd.DeviceState = deviceState;
            m_RequestDispatcher->SendMessage(m_sModuleId.c_str(), cmd);
          }
          
        END_WRAPPER();
        
        BEGIN_IMPLEMENTATION()
          /// <summary> 
          ///   GetState 
          /// </summary>
          /// <param name="device"></param>
          void SendGetState(address target, const string & device)
          {
            G1::GetState cmd;
            cmd.Device = device;
            m_RequestDispatcher->SendMessage(target, cmd);
          }
          /// <summary> 
          ///   GetState  
          /// </summary>
          /// <param name="device"></param>
          void RaiseGetState(const string & device)
          {
            G1::GetState cmd;
            cmd.Device = device;
            m_RequestDispatcher->SendMessage(m_sModuleId.c_str(), cmd);
          }
          
          /// <summary> 
          ///   SetState 
          /// </summary>
          /// <param name="device"></param>
          /// <param name="deviceState"></param>
          void SendSetState(address target, const string & device, const string & deviceState)
          {
            G1::SetState cmd;
            cmd.Device = device;
            cmd.DeviceState = deviceState;
            m_RequestDispatcher->SendMessage(target, cmd);
          }
          /// <summary> 
          ///   SetState  
          /// </summary>
          /// <param name="device"></param>
          /// <param name="deviceState"></param>
          void RaiseSetState(const string & device, const string & deviceState)
          {
            G1::SetState cmd;
            cmd.Device = device;
            cmd.DeviceState = deviceState;
            m_RequestDispatcher->SendMessage(m_sModuleId.c_str(), cmd);
          }
          
        END_IMPLEMENTATION();
        
        #pragma endregion
      END_PROTOCOL()
      
      } //   Protocol
    } //   CrossroadController
  } //   Uniteller
